#ifndef cdfur_H
#define cdfur_H

#include "cdstrand.h"
#include <stdio.h>

//---------------------------------------------------------------------------------------
class CDFur
{

private:

   // Get the rotation of a strand based on it's current direction.
   // Rotation is expressed in radians, and assumes that a 2-sided XZ profile
   // will be the target rotated.
   float Get2SidedRotationXZ( CDStrand *s, float defval );

   // Determines whether a strand already exists on an adjoining surface in the desired location
   CDStrand *FindDuplicateStrand( Vertex *v1, Vertex *v2, Vertex *v3, Vertex *strandloc );

public:

   int mDebugMode;

   // Randomly offset each hair on the surface.
   // This offset is between 0 and 1, but represents the % shift the hair can
   // be offset up to 0.5 the distance to the next hair.
   float mRandomPlacement;

   // Randomly alter the direction of the fur.
   float mRandomDirection;

   // Number of strands per unit.
   // This is the number of strands in one direction --- so, 10 strands per unit means
   // 10 strands will be side-by-side across one unit, and a 1x1 square will contain 100 strands.
   float mStrandsPerUnit;

   // Thickness at the base of each strand.
   float mStrandThickness;
   // Random scaling (percentage +/-) applied to the thickness of each strand
   float mRandomStrandThickness;

   // Segments in the fur
   int mSegments;

   // Length of the fur
   float mLength;
   // Random scaling (percentage +/-) applied to the length of each strand
   float mRandomLength;

   // Remove duplicates
   bool mRemoveDuplicates;

   // Whether to apply random rotation to each hair before placing.  If false,
   // rotate each hair in accordance with hair's direction from root to the end
   // of the first segment.
   bool mRotateStrands;

   // Random seed applied to all randomized attributes.
   unsigned int mRandomSeed;

   // Axis-aligned gravity
   Vertex mGravity;

   // Number of sides in hair polygons (1 = line)
   int mStrandSides;

   // Maximum number of hairs
   unsigned int mMaxStrands;

   // Depth of hair embedded under surface
   float mRootDepth;

   // Strands
   CDStrand *mSHead;
   CDStrand *mSTail;

   // Constructor (private for singleton class)
   CDFur();
   // Destructor
   ~CDFur();

   // Generate fur
   ACObject *Generate( bool fur_add_undoable );

   // Check input settings - return false if a value is out of bounds.
   // Displays a message to the user on failure
   bool CheckInputs( bool console_only );

   // Print settings
   void DebugPrintSettings();

};
//---------------------------------------------------------------------------------------

#endif
