#ifndef cdstrand_H
#define cdstrand_H

#include "c3d.h"

//---------------------------------------------------------------------------------------
// CDStrand : Geometry used to create a strand of fur.
//
// Some functions (noted in comments above each function) were stringly derived
//    or outright copied from Andy Thaller's Extrude Along Path plugin for AC3D.
// This code was used in accordance with the GNU General Public License.
// For more information on Andy Thaller's work, see http://www.ph.tum.de/~thaller/extrude/.
//
class CDStrand
{

private:

   // Number of segments
   unsigned short mNumSegments;
   // Number of extruded vertices
   unsigned short mNumExtVerts;

public:

   // Next strand in the list
   CDStrand *mNext;

   // Root location
   Vertex mRoot;
   // AC3D Surface in which this strand is rooted
   Surface *mParentSurface;
   // Triangle vertices between which this strand was applied
   Vertex mDirection;

   // Segment end points
   Vertex *mSegmentVerts;
   // Segment direction points
   Vertex *mSegmentDirs;

   // Strand extruded from 2d profile
   Vertex *mExtVerts;

   int mDebug;

   // Constructor
   CDStrand( unsigned short numsegments );
   // Destructor
   ~CDStrand();

   // Cleanup the line mSegmentVerts and mSegmentDirs objects
   void ClearSegments();
   // Cleanup the line mExtVerts object
   void ClearExtruded();

   // Create an extruded strand object
   // NOTE: This code is an adaptation of Andy Thaller's Extrude Along Path plugin.
   void CreateExtrudedStrand( int numverts, float diameter, float rotprofangle );

};
//---------------------------------------------------------------------------------------

#endif
