#include "cdstrandprofile.h"

//---------------------------------------------------------------------------------------
CDStrandProfile::CDStrandProfile( unsigned short numverts )
{
   mNumVerts = numverts;
   mVerts = new Vertex[mNumVerts];
}
//---------------------------------------------------------------------------------------
CDStrandProfile::~CDStrandProfile()
{
   if( mVerts != NULL )
   {
      delete[] mVerts;
   }
}
//---------------------------------------------------------------------------------------
void CDStrandProfile::CreateStrandProfileXZ( float diameter, float rotaterads )
{
   // Plot point 1 along the X axis
   mVerts[0].x = diameter * 0.5f;
   mVerts[0].y = mVerts[0].z = 0.0f;
   // Apply initial rotation
   if( rotaterads != 0 )
   {
      float v0 = mVerts[0].x;
      float v1 = mVerts[0].z;
      mVerts[0].x = cos( rotaterads ) * v0 - sin( rotaterads ) * v1;
      mVerts[0].z = sin( rotaterads ) * v0 + cos( rotaterads ) * v1;
   }
   // Determine the rotation degrees
   float theta = DEGTORAD( 360.0f / ( float )mNumVerts );
   // Plot the remaining points around the Y axis (rotate around origin)
   for( int ix = 1; ix < mNumVerts; ++ix )
   {
      mVerts[ix].x = cos( theta ) * mVerts[ix-1].x - sin( theta ) * mVerts[ix-1].z;
      mVerts[ix].y = 0;
      mVerts[ix].z = sin( theta ) * mVerts[ix-1].x + cos( theta ) * mVerts[ix-1].z;
   }
}
//---------------------------------------------------------------------------------------
void CDStrandProfile::MoveBy( Vertex *delta )
{
   for( int ix = 0; ix < mNumVerts; ++ix )
   {
      mVerts[ix].x += delta->x;
      mVerts[ix].y += delta->y;
      mVerts[ix].z += delta->z;
   }
}
//---------------------------------------------------------------------------------------
void CDStrandProfile::RotateAxis( Vertex *center, Vertex *axis, double C )
{
   double S;
   double a = axis->x,
          b = axis->y,
          c = axis->z;
   double x0 = center->x,
          y0 = center->y,
          z0 = center->z;
   if( C < 0.0 )
   {
      C = 0.0;
   }
   else if( C > 1.0 )
   {
      C = 1.0;
   }
   S = sqrt( 1.0 - C * C );
   for( int ix = 0; ix < mNumVerts; ++ix )
   {
      Vertex *v = &( mVerts[ix] );
      double x = v->x - x0,
             y = v->y - y0,
             z = v->z - z0;
      v->x = ( float )((( 1.0 - C ) * a * a + C     ) * x + (( 1.0 - C ) * b * a - c * S ) * y + (( 1.0 - C ) * c * a + b * S ) * z + x0 );
      v->y = ( float )((( 1.0 - C ) * a * b + c * S ) * x + (( 1.0 - C ) * b * b + C     ) * y + (( 1.0 - C ) * c * b - a * S ) * z + y0 );
      v->z = ( float )((( 1.0 - C ) * a * c - b * S ) * x + (( 1.0 - C ) * b * c + a * S ) * y + (( 1.0 - C ) * c * c + C     ) * z + z0 );
   }
}
//---------------------------------------------------------------------------------------
void CDStrandProfile::GetIntersections( CDStrandProfile *p1, CDStrandProfile *p2, CDStrandProfile *p3, CDStrandProfile *p4 )
{
   for( int ix = 0; ix < mNumVerts; ++ix )
   {
      Vertex *v1 = &( p1->mVerts[ix] );
      Vertex *v2 = &( p2->mVerts[ix] );
      Vertex *v3 = &( p3->mVerts[ix] );
      Vertex *v4 = &( p4->mVerts[ix] );
      C3d::vtx_line_intersection( mVerts[ix], *v1, *v2, *v3, *v4 );
   }
}
//---------------------------------------------------------------------------------------
void CDStrandProfile::ReverseVertexOrder()
{
   CDStrandProfile temp( mNumVerts );
   temp.CopyFrom( this );
   for( int ix = 0; ix < mNumVerts; ++ix )
   {
      memcpy( &( mVerts[ix] ), &( temp.mVerts[mNumVerts - ( ix + 1 )] ), sizeof( Vertex ));
   }
}
//---------------------------------------------------------------------------------------
