#ifndef cdstrandprofile_H
#define cdstrandprofile_H

#include "c3d.h"

//---------------------------------------------------------------------------------------
// CDStrandProfile : Extrude-able profile for a polygonal strand of fur.
//
// Some functions (noted in comments above each function) were stringly derived
//    or outright copied from Andy Thaller's Extrude Along Path plugin for AC3D.
// This code was used in accordance with the GNU General Public License.
// For more information on Andy Thaller's work, see http://www.ph.tum.de/~thaller/extrude/.
//
class CDStrandProfile
{

private:

   // Number of vertices
   unsigned short mNumVerts;

public:

   // Segment end points
   Vertex *mVerts;

   // Constructor
   CDStrandProfile( unsigned short numverts );
   // Destructor
   ~CDStrandProfile();

   // Create a 2d strand profile on the XZ axis
   void CreateStrandProfileXZ( float diameter, float rotaterads );

   // Copy the profile
   void CopyFrom( CDStrandProfile *src )
   {
      this->mNumVerts = src->mNumVerts;
      memcpy( this->mVerts, src->mVerts, sizeof( Vertex ) * this->mNumVerts );
   }

   // Move all vertices in the profile by the specified delta
   void MoveBy( Vertex *delta );
   // Rotate all vertices in the profile by the specified axis
   // NOTE: This code is an adaptation of function(s) in Andy Thaller's Extrude Along Path plugin.
   void RotateAxis( Vertex *center, Vertex *axis, double C );

   // Get the point intersection between profiles
   // NOTE: This code is an adaptation of function(s) in Andy Thaller's Extrude Along Path plugin.
   void GetIntersections( CDStrandProfile *p1, CDStrandProfile *p2, CDStrandProfile *p3, CDStrandProfile *p4 );

   // Reverse the vertex ordering
   void ReverseVertexOrder();

};
//---------------------------------------------------------------------------------------

#endif
